/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.1, (the "License"); you may not use this file except in
 * compliance with the License. You should have received a copy of the
 * Erlang Public License along with this software. If not, it can be
 * retrieved via the world wide web at http://www.erlang.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Original Code is Ericsson Utvecklings AB.
 * Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
 * AB. All Rights Reserved.''
 * 
 *     $Id: benchmark.c,v 1.1 2002/02/08 15:11:33 jewi7435 Exp $
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "sys.h"
#include "global.h"
#include "benchmark.h"
#include <time.h>

#ifdef __BENCHMARK__


Uint   processes_spawned;
double messages_sent;
double major_garbage_cols;
double minor_garbage_cols;
Uint   biggest_heap_size_ever;
Uint   max_allocated_heap;
double live_major_sum;
double live_minor_sum;
double ptrs_to_old;
double ptrs_to_young;

hrtime_t system_time;
hrtime_t minor_gc_time;
hrtime_t major_gc_time;
hrtime_t send_time;
hrtime_t copy_time;
hrtime_t max_major_time;
hrtime_t max_minor_time;

#ifdef CALCULATE_MESSAGE_SIZES
Uint message_sizes[1000];
#endif


__inline__ void init_benchmarking()
{
    minor_gc_time = 0;
    major_gc_time = 0;
    send_time = 0;
    copy_time = 0;
    max_major_time = 0;
    max_minor_time = 0;
    major_garbage_cols = 0;
    minor_garbage_cols = 0;
    biggest_heap_size_ever = 0;
    max_allocated_heap = 0;
    live_major_sum = 0;
    live_minor_sum = 0;
    ptrs_to_old = 0;
    ptrs_to_young = 0;
    processes_spawned = 0;
    messages_sent = 0;
#ifdef CALCULATE_MESSAGE_SIZES
    { int i;
      for (i=0; i<1000; i++) message_sizes[i] = 0;
    }
#endif
#ifdef NO_IDLE_TIME
    system_time = 0;
#else
    system_time = sys_gethrtime();
#endif
}

static void extract_nodename(char *str)
{
    int i = 1;
    while (str[i] != '\0' && str[i] != '\'') i++;
    if (str[i] == '\'') str[i+1] = '\0';
}

__inline__ void save_statistics()
{
    FILE *file = fopen(STATISTICS_FILE,"a");

    if (file)
    {
#ifdef NO_IDLE_TIME
        hrtime_t sys_time = system_time;
#else
        hrtime_t sys_time = sys_gethrtime() - system_time;
#endif

        ldisplay(this_node,CBUF,50);
        extract_nodename(tmp_buf);
        fprintf(file,tmp_buf);
        fprintf(file,"-------------------------------------------------------------------------\n");
        fprintf(file,"The counters are reset at system start and are sums over the entire node.\n");
        fprintf(file,"You may reset them manually using the clear BIFs in the module hipe_bifs.\n");
        fprintf(file,"All times are given in milliseconds.\n");
        fprintf(file,"-------------------------------------------------------------------------\n");
        fprintf(file,"Number of processes spawned: %ld\n",processes_spawned);
        fprintf(file,"Number of messages sent: %.0f\n",messages_sent);
        fprintf(file,"Biggest heap used (in words): %ld\n",biggest_heap_size_ever);
        fprintf(file,"Biggest heap allocated (in words): %ld\n",max_allocated_heap);
        fprintf(file,"Number of minor garbage collections: %.0f\n",minor_garbage_cols);
        fprintf(file,"Number of major garbage collections: %.0f\n",major_garbage_cols);
        fprintf(file,"Live data recovered in minor collections (in words): %.0f\n",live_minor_sum);
        fprintf(file,"Live data recovered in major collections (in words): %.0f\n",live_major_sum);
        fprintf(file,"Number of pointers to young generation: %.0f\n",ptrs_to_young);
        fprintf(file,"Number of pointers to old generation: %.0f\n",ptrs_to_old);
        TIME_PRINTER("Total system time: %d:%02d.%03d\n",sys_time);
        TIME_PRINTER("Time spent in send (including copy): %d:%02d.%03d\n",send_time);
        TIME_PRINTER("Time spent in copy: %d:%02d.%03d\n",copy_time);
        TIME_PRINTER("Time spent in minor garbage collection: %d:%02d.%03d\n",minor_gc_time);
        TIME_PRINTER("Time spent in major garbage collection: %d:%02d.%03d\n",major_gc_time);
        TIME_PRINTER("Maximum time spent in one separate minor garbage collection: %d:%02d.%03d\n",max_minor_time);
        TIME_PRINTER("Maximum time spent in one separate major garbage collection: %d:%02d.%03d\n",max_major_time);
        fclose(file);
    }
    else
        printf("Sorry... Can not write to %s!\n\r",STATISTICS_FILE);
}
#else
__inline__ void init_benchmarking() { }
__inline__ void save_statistics()   { }
#endif
