%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(dbg_ui_backtrace).

%% External exports
-export([start/3, start/4]).

%% Internal exports
-export([init/5]).

-record(state, {win,    % term() Backtrace size dialog window data
		pid     % pid() Parent
	       }).

%%====================================================================
%% External exports
%%====================================================================

%%--------------------------------------------------------------------
%% start(GS, Pos, BackTrace)
%% start(GS, Pos, Pid, BackTrace)
%%   GS = graphics system identifier
%%   Pos = {X, Y}
%%     X = Y = integer()
%%   Pid = pid() Parent
%%   BackTrace = integer()
%%--------------------------------------------------------------------
start(GS, Pos, BackTrace) ->
    start2(GS, Pos, "Back Trace Size", BackTrace).
start(GS, Pos, Pid, BackTrace) ->
    Title = "Back Trace Size " ++ pid_to_list(Pid),
    start2(GS, Pos, Title, BackTrace).

start2(GS, Pos, Title, BackTrace) ->
    case dbg_ui_winman:is_started(Title) of
	true -> ignore;
	false ->
	    spawn_link(?MODULE, init, [self(), GS, Pos, Title, BackTrace])
    end.


%%====================================================================
%% Internal exports
%%====================================================================

init(Pid, GS, Pos, Title, BackTrace) ->

    %% Create backtrace dialog window
    Win = dbg_ui_backtrace_win:create_win(GS, Pos, Title, BackTrace),
    Window = dbg_ui_backtrace_win:get_window(Win),
    dbg_ui_winman:insert(Title, Window),
    State = #state{win=Win, pid=Pid},

    loop(State).

loop(State) ->
    receive

	%% From the GUI
	GuiEvent when tuple(GuiEvent), element(1, GuiEvent)==gs ->
	    Cmd = dbg_ui_backtrace_win:handle_event(GuiEvent,
						    State#state.win),
	    State2 = gui_cmd(Cmd, State),
	    loop(State2);

	%% From the dbg_ui_winman process (Debugger window manager)
	%% Ignore them as the window contains no Window menu and
	%% the process is linked to (and thus teared down by) its parent
	{dbg_ui_winman, update_windows_menu, Data} ->
	    loop(State);
	{dbg_ui_winman, destroy} ->
	    loop(State)
    end.

gui_cmd(ignore, State) ->
    State;
gui_cmd(stopped, State) ->
    exit(normal);
gui_cmd({backtrace, BackTrace}, State) ->
    State#state.pid ! {dbg_ui_backtrace, BackTrace},
    exit(normal).
