%% -*- erlang-indent-level: 2 -*-
%% =====================================================================
%% Copyright (c) 1997 by the HiPE group.  All Rights Reserved 
%% =====================================================================
%%  Filename : 	hipe_sparc.erl
%%  Group    :  Optimizer
%%  Purpose  :  Primitives for handling sparc three address instructions
%%  Notes    : 
%% =====================================================================
%% Exported functions (short description):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-module(hipe_sparc).

-export([mk_sparc/8,
	 mk_sparc/6,
	 sparc_fun/1,
	 sparc_arity/1,
	 sparc_is_closure/1,
	 sparc_is_leaf/1,
	 sparc_code/1,
	 sparc_code_update/2,
	 sparc_data/1,
	 sparc_data_update/2,
	 sparc_var_range/1,
	 sparc_var_range_update/2,
	 sparc_label_range/1,
	 sparc_label_range_update/2,
	 sparc_size/1]).

-export([store_create/5,
	 store_dest/1,
	 store_dest_update/2,
	 store_off/1,
	 store_off_update/2,
	 store_src/1,
	 store_src_update/2,
	 store_type/1,
	 store_type_update/2]).

-export([jmp_link_create/5,
	 jmp_link_off/1,
	 jmp_link_off_update/2,
	 jmp_link_target/1,
	 jmp_link_target_update/2,
	 jmp_link_args/1,
	 jmp_link_args_update/2,
	 jmp_link_link/1,
	 jmp_link_link_update/2,

	 jmp_create/4,
	 jmp_create/5,
	 jmp_off/1,
	 jmp_off_update/2,
	 jmp_target/1,
	 jmp_target_update/2,
	 jmp_args/1,
	 jmp_args_update/2,
	 jmp_destinations/1,
	 jmp_destinations_update/2,

	 call_link_create/7,
	 call_link_target/1,
	 call_link_target_update/2,
	 call_link_link/1,
	 call_link_link_update/2,
	 call_link_args/1,
	 call_link_args_update/2,
	 call_link_dests/1,
	 call_link_continuation/1,
	 call_link_continuation_update/2,
	 call_link_fail/1,
	 call_link_fail_update/2,
	 call_link_type/1,
	 call_link_stack_desc/1,
	 call_link_stack_desc_update/2,

	 label_create/2,
	 label_create_new/0,
	 label_name/1,
	 label_name_update/2,

	 load_atom_create/3,
	 load_atom_dest/1,
	 load_atom_dest_update/2,
	 load_atom_atom/1,

	 load_word_index_create/4,
	 load_word_index_dest/1,
	 load_word_index_dest_update/2,
	 load_word_index_index/1,
	 load_word_index_block/1,

	 load_address_create/4,
	 load_address_dest/1,
	 load_address_address/1,
	 load_address_address_update/2,
	 load_address_type/1,
	 load_address_type_update/2,
	 load_address_dest_update/2,

	 load_create/5,
	 load_dest/1,
	 load_dest_update/2,
	 load_off/1,
	 load_off_update/2,
	 load_src/1,
	 load_src_update/2,
	 load_type/1,
	 load_type_update/2,

	 move_create/3,
	 move_dest/1,
	 move_dest_update/2,
	 move_src/1,
	 move_src_update/2,

	 multimove_create/3,
	 multimove_dest/1,
	 multimove_dest_update/2,
	 multimove_src/1,
	 multimove_src_update/2,

	 nop_create/1,

	 sethi_const/1,
	 sethi_const_update/2,
	 sethi_create/3,
	 sethi_dest/1,
	 sethi_dest_update/2]).


-export([
	 pseudo_return_create/1,
	 pseudo_return_create/2,
	 pseudo_return_regs/1,
	 pseudo_return_regs_update/2,

	 pseudo_enter_create/3,
	 pseudo_enter_create/4,
	 pseudo_enter_target/1,
	 pseudo_enter_target_update/2,
	 pseudo_enter_type/1,
	 pseudo_enter_args/1,
	 pseudo_enter_args_update/2,

	 pseudo_spill_create/2,
	 pseudo_spill_create/3,
	 pseudo_spill_reg/1,
	 pseudo_spill_pos/1,
	 pseudo_unspill_create/2,
	 pseudo_unspill_create/3,
	 pseudo_unspill_reg/1,
	 pseudo_unspill_pos/1,

	 pseudo_push_create/1,
	 pseudo_push_create/2,
	 pseudo_push_reg/1,
	 pseudo_push_reg_update/2,
	 pseudo_pop_create/2,
	 pseudo_pop_create/3,
	 pseudo_pop_reg/1,
	 pseudo_pop_index/1,
	 pseudo_pop_reg_update/2]).

-export([align_alignment/1,
	 align_alignment_update/2,
	 align_create/2,

	 alu_cc_create/5,
	 alu_cc_dest/1,
	 alu_cc_dest_update/2, 
	 alu_cc_operator/1,
	 alu_cc_operator_update/2,
	 alu_cc_src1/1, 
	 alu_cc_src1_update/2,
	 alu_cc_src2/1,
	 alu_cc_src2_update/2,

	 alu_create/5,
	 alu_dest/1,
	 alu_dest_update/2,
	 alu_operator/1,
	 alu_operator_update/2,
	 alu_src1/1,
	 alu_src1_update/2,
	 alu_src2/1,
	 alu_src2_update/2, 

	 b_create/6,
	 b_annul/1,
	 b_annul_update/2,
	 b_cond/1,
	 b_cond_update/2,
	 b_label/1,
	 b_label_update/2,
	 b_true_label/1,
	 b_true_label_update/2,
	 b_false_label/1,
	 b_false_label_update/2,
	 b_pred/1,
	 b_pred_update/2,
	 b_taken/1,

	 br_create/7,
	 br_annul/1,
	 br_annul_update/2,
	 br_label/1,
	 br_label_update/2,
	 br_true_label/1,
	 br_true_label_update/2,
	 br_false_label/1,
	 br_false_label_update/2,
	 br_pred/1,
	 br_pred_update/2,
	 br_taken/1,
	 br_reg/1,
	 br_reg_update/2,
	 br_regcond/1,
	 br_regcond_update/2,

	 goto_create/2,
	 goto_label/1,
	 goto_label_update/2,
	 is_goto/1,

	 cmov_cc_cond/1,
	 cmov_cc_cond_update/2,
	 cmov_cc_create/4,
	 cmov_cc_dest/1,
	 cmov_cc_dest_update/2,
	 cmov_cc_src/1,
	 cmov_cc_src_update/2,

	 cmov_r_create/5,
	 cmov_r_dest/1,
	 cmov_r_dest_update/2,
	 cmov_r_reg/1,
	 cmov_r_reg_update/2,
	 cmov_r_regcond/1,
	 cmov_r_regcond_update/2,
	 cmov_r_src/1,
	 cmov_r_src_update/2,
	 comment_create/2,

	 comment_text/1,
	 comment_text_update/2,

	 info/1,
	 info_update/2,
	 is_align/1,
	 is_alu/1,
	 is_alu_cc/1,
	 is_any_alu/1,
	 is_any_branch/1,
	 is_any_cmov/1,
	 is_any_memop/1,
	 is_b/1,
	 is_br/1,
	 is_call_link/1,
	 is_cmov_cc/1,
	 is_cmov_r/1,
	 is_comment/1,
	 is_jmp/1,
	 is_jmp_link/1,
	 is_label/1,
	 is_load/1,
	 is_move/1,
	 is_multimove/1,
	 is_nop/1,
	 is_sethi/1,
	 is_store/1]).

-export([redirect_jmp/3,
	 cc_negate/1,
	 uses/1,
	 all_uses/1,
	 imm_uses/1,
	 defines/1,
	 def_use/1,
	 subst/2,
	 subst_uses/2,
	 subst_defines/2]).


-export([mk_reg/1,
	 mk_new_reg/0,
	 is_reg/1,
	 reg_nr/1,
	 mk_fpreg/1,
	 is_fpreg/1,
	 fpreg_nr/1,
	 mk_imm/1,
	 is_imm/1,
	 imm_value/1,
	 mk_spill/1,
	 is_spill/1,
	 spill_pos/1
]).
-export([type/1]).
-export([highest_reg/1]).


-include("hipe_sparc.hrl").

%-define(DO_ASSERT,true).
-include("../main/hipe.hrl").



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Constructors
%%

mk_sparc(Fun, Arity, Closure, Leaf, Code, Data, VarRange, LabelRange) ->
  #sparc{'fun'=Fun, 
	 arity=Arity, closure=Closure, leaf=Leaf,
	 code=Code, data=Data, var_range=VarRange, label_range=LabelRange}.
mk_sparc(Fun, Arity, Code, Data, VarRange, LabelRange) ->
  #sparc{'fun'=Fun, 
	 arity=Arity, closure=false, leaf=false,
	 code=Code, data=Data, var_range=VarRange, label_range=LabelRange}.
sparc_fun(Sparc) -> Sparc#sparc.'fun'.
sparc_arity(Sparc) -> Sparc#sparc.arity.
sparc_is_closure(Sparc) -> Sparc#sparc.closure.
sparc_is_leaf(Sparc) -> Sparc#sparc.leaf.  
sparc_code(Sparc) -> Sparc#sparc.code.
sparc_code_update(Sparc, NewCode) -> Sparc#sparc{code=NewCode}.
sparc_data(Sparc) -> Sparc#sparc.data.
sparc_data_update(Sparc, NewData) -> Sparc#sparc{data=NewData}.
sparc_var_range(Sparc) -> Sparc#sparc.var_range.
sparc_var_range_update(Sparc, NewRange) -> 
  NewSparc = Sparc#sparc{var_range=NewRange},
  ?VERBOSE_ASSERT(check_var_range(NewSparc)),
  NewSparc.

sparc_label_range(Sparc) -> Sparc#sparc.label_range.
sparc_label_range_update(Sparc, NewRange) -> Sparc#sparc{label_range=NewRange}.
sparc_size(Sparc) ->
  RealIns = fun(I) -> not(is_label(I) or is_comment(I)) end,
  length(lists:filter(RealIns, sparc_code(Sparc))).



info_update(Insn, NewInfo) ->
  if
    record(Insn,label) -> Insn#label{info=NewInfo};
    record(Insn,nop) -> Insn#nop{info=NewInfo};
    record(Insn,align) -> Insn#align{info=NewInfo};
    record(Insn,comment) -> Insn#comment{info=NewInfo};
    record(Insn,move) -> Insn#move{info=NewInfo};
    record(Insn,multimove) -> Insn#multimove{info=NewInfo};
    record(Insn,cmov_cc) -> Insn#cmov_cc{info=NewInfo};
    record(Insn,cmov_r) -> Insn#cmov_r{info=NewInfo};
    record(Insn,alu) -> Insn#alu{info=NewInfo};
    record(Insn,alu_cc) -> Insn#alu_cc{info=NewInfo};
    record(Insn,sethi) -> Insn#sethi{info=NewInfo};
    record(Insn,load) -> Insn#load{info=NewInfo};
    record(Insn,store) -> Insn#store{info=NewInfo};
    record(Insn,b) -> Insn#b{info=NewInfo};
    record(Insn,br) -> Insn#br{info=NewInfo};
    record(Insn,jmp_link) -> Insn#jmp_link{info=NewInfo};
    record(Insn,jmp) -> Insn#jmp{info=NewInfo};
    record(Insn,call_link) -> Insn#call_link{info=NewInfo};
    record(Insn,load_atom) -> Insn#load_atom{info=NewInfo};
    record(Insn,load_word_index) -> Insn#load_word_index{info=NewInfo};
    record(Insn,load_address) -> Insn#load_address{info=NewInfo};
    record(Insn,pseudo_pop) -> Insn#pseudo_pop{info=NewInfo};
    record(Insn,pseudo_push) -> Insn#pseudo_push{info=NewInfo};
    record(Insn,pseudo_enter) -> Insn#pseudo_enter{info=NewInfo};
    record(Insn,pseudo_return) -> Insn#pseudo_return{info=NewInfo}
  end.


%% Enter - pseudo op %
pseudo_enter_create(Target, Args, Type) ->
  #pseudo_enter{target=Target, args=Args, type=Type}.
pseudo_enter_create(Target, Args, Type, Info) ->
  #pseudo_enter{target=Target, args=Args, type=Type, info=Info}.
pseudo_enter_args(I) ->
  I#pseudo_enter.args.
pseudo_enter_args_update(I,Args) ->
  I#pseudo_enter{args=Args}.
pseudo_enter_target(I) ->
  I#pseudo_enter.target.
pseudo_enter_target_update(I,Target) ->
  I#pseudo_enter{target=Target}.
pseudo_enter_type(I) ->
  I#pseudo_enter.type.



%% Return - pseudo op %
pseudo_return_create(Regs) ->
  #pseudo_return{regs=Regs}.
pseudo_return_create(Regs, Info) ->
  #pseudo_return{regs=Regs, info=Info}.
pseudo_return_regs(I) ->
  I#pseudo_return.regs.
pseudo_return_regs_update(I,Regs) ->
  I#pseudo_return{regs=Regs}.


%% Push - pseudo op %
pseudo_push_create(Reg) ->
  #pseudo_push{reg=Reg}.
pseudo_push_create(Reg, Info) ->
  #pseudo_push{reg=Reg, info=Info}.
pseudo_push_reg(I) ->
  I#pseudo_push.reg.
pseudo_push_reg_update(I,Reg) ->
  I#pseudo_push{reg=Reg}.

%% Pop - pseudo op %
pseudo_pop_create(Reg, Index) ->
  #pseudo_pop{reg=Reg, index=Index}.
pseudo_pop_create(Reg, Index, Info) ->
  #pseudo_pop{reg=Reg, index=Index, info=Info}.
pseudo_pop_reg(I) ->
  I#pseudo_pop.reg.
pseudo_pop_index(I) ->
  I#pseudo_pop.index.
pseudo_pop_reg_update(I,Reg) ->
  I#pseudo_pop{reg=Reg}.

%% spill - pseudo op %
pseudo_spill_create(Reg1,Pos) ->
  #pseudo_spill{source=Reg1, dest=Pos}.
pseudo_spill_create(Reg1, Reg2, Info) ->
  #pseudo_spill{source=Reg1, dest=Reg2, info=Info}.
pseudo_spill_reg(I) ->
  I#pseudo_spill.source.
pseudo_spill_pos(I) ->
  I#pseudo_spill.dest.
pseudo_spill_reg_update(I,Reg) ->
  I#pseudo_spill{source=Reg}.

%% unspill - pseudo op %
pseudo_unspill_create(Reg1, Pos) ->
  #pseudo_unspill{source=Reg1, dest=Pos}.
pseudo_unspill_create(Reg1, Pos, Info) ->
  #pseudo_unspill{source=Reg1, dest=Pos, info=Info}.
pseudo_unspill_reg(I) ->
  I#pseudo_unspill.source.
pseudo_unspill_pos(I) ->
  I#pseudo_unspill.dest.
pseudo_unspill_reg_update(I,Reg) ->
  I#pseudo_unspill{source=Reg}.


%% Load atom %%
load_atom_create(Dest,Atom,Info) -> #load_atom{dst=Dest,atom=Atom,info=Info}.
load_atom_dest(LoadAtom) -> LoadAtom#load_atom.dst.
load_atom_atom(LoadAtom) -> LoadAtom#load_atom.atom.
load_atom_dest_update(LA, NewDest) -> LA#load_atom{dst=NewDest}.

load_word_index_create(Dest,Block,Index,Info) -> #load_word_index{dst=Dest, block = Block, index=Index, info=Info}.
load_word_index_dest(LoadAtom) ->LoadAtom#load_word_index.dst.
load_word_index_dest_update(LA, NewDest) -> LA#load_word_index{dst=NewDest}.
load_word_index_block(LoadAtom) ->LoadAtom#load_word_index.block.
load_word_index_index(LoadAtom) ->LoadAtom#load_word_index.index.

%%
%% Load address
%%

load_address_create(Dst, Addr, Type, Info) -> 
  #load_address{dst=Dst, address=Addr, type=Type, info=Info}.
load_address_dest(LoadAddress) -> LoadAddress#load_address.dst.
load_address_address(LoadAddress) -> LoadAddress#load_address.address.
load_address_address_update(LoadAddress, NewA) -> LoadAddress#load_address{address=NewA}.
load_address_dest_update(LA, NewDst) -> LA#load_address{dst=NewDst}.
load_address_type(LoadAddress) -> LoadAddress#load_address.type.
load_address_type_update(LA, NewType) -> LA#load_address{type=NewType}.


%% 
%% label
%%

label_create(Name,Info) -> #label{id=Name,info=Info}.
label_create_new() -> label_create(hipe_gensym:get_next_label(sparc), []).
label_name(Label) -> Label#label.id.
label_name_update(Label,NewName) -> Label#label{id=NewName}.
is_label(Insn) -> case type(Insn) of label -> true; _ -> false end.


%%
%% nop
%%

nop_create(Info) -> #nop{info=Info}.
is_nop(Insn) -> case type(Insn) of nop -> true; _ -> false end.


%% Align %%
align_create(Number,Info) -> #align{alignment=Number,info=Info}.
align_alignment(Align) -> Align#align.alignment.
align_alignment_update(Align,NewNumber) -> Align#align{alignment=NewNumber}.
is_align(Insn) -> case type(Insn) of align -> true; _ -> false end.

%% Comment %%
comment_create(Text, Info) -> #comment{text=Text,info=Info}.
comment_text(Comment) -> Comment#comment.text.
comment_text_update(Comment, NewText) -> Comment#comment{text=NewText}.
is_comment(Insn) -> case type(Insn) of  comment -> true; _ -> false end.

%% Move %%
move_create(Dest, Source, Info) -> #move{dst=Dest, src=Source, info=Info}.
move_dest(Move) -> Move#move.dst.
move_src(Move) -> Move#move.src.
move_dest_update(Move, NewDest) -> Move#move{dst=NewDest}.
move_src_update(Move, NewSource) -> Move#move{src=NewSource}.
is_move(Insn) -> case type(Insn) of  move -> true; _ -> false end.

%% MultiMove %%
multimove_create(Dest, Source, Info) -> 
  case length(Dest) =:= length(Source) of
    true -> true;
    false -> exit({sparc, multimove, src_dst_aritydiff})
  end,
  #multimove{dst=Dest, src=Source, info=Info}.
multimove_dest(Move) -> Move#multimove.dst.
multimove_src(Move) -> Move#multimove.src.
multimove_dest_update(Move, NewDest) -> Move#multimove{dst=NewDest}.
multimove_src_update(Move, NewSource) -> Move#multimove{src=NewSource}.
is_multimove(Insn) -> case type(Insn) of  multimove -> true; _ -> false end.

%% Cmov_cc %%
cmov_cc_create(Dest,Source,Cond,Info) -> 
  #cmov_cc{dst=Dest,src=Source,cc=Cond,info=Info}.
cmov_cc_dest(CmovCc) -> CmovCc#cmov_cc.dst.
cmov_cc_src(CmovCc) -> CmovCc#cmov_cc.src.
cmov_cc_cond(CmovCc) -> CmovCc#cmov_cc.cc.
cmov_cc_dest_update(CmovCc,NewDest) -> CmovCc#cmov_cc{dst=NewDest}.
cmov_cc_src_update(CmovCc,NewSource) -> CmovCc#cmov_cc{src=NewSource}.
cmov_cc_cond_update(CmovCc,NewCond) -> CmovCc#cmov_cc{cc=NewCond}.

%% Cmov_r %%
cmov_r_create(Dest,Source,Reg,RegCond,Info) -> 
  #cmov_r{dst=Dest,src=Source,rcc=RegCond,reg=Reg,info=Info}.
cmov_r_dest(CmovR) -> CmovR#cmov_r.dst.
cmov_r_src(CmovR) -> CmovR#cmov_r.src.
cmov_r_reg(CmovR) -> CmovR#cmov_r.reg.
cmov_r_regcond(CmovR) -> CmovR#cmov_r.rcc.
cmov_r_dest_update(CmovR,NewDest) -> CmovR#cmov_r{dst=NewDest}.
cmov_r_src_update(CmovR,NewSource) -> CmovR#cmov_r{src=NewSource}.
cmov_r_reg_update(CmovR,NewReg) -> CmovR#cmov_r{reg=NewReg}.
cmov_r_regcond_update(CmovR,NewRegCond) -> CmovR#cmov_r{rcc=NewRegCond}.
is_cmov_r(Insn) -> case type(Insn) of  cmov_r -> true; _ -> false end.

%% Alu %%
alu_create(Dest,Src1,Op,Src2,Info) -> 
  #alu{dst=Dest,src1=Src1,src2=Src2,op=Op,info=Info}.
alu_dest(Alu) -> Alu#alu.dst.
alu_src1(Alu) -> Alu#alu.src1.
alu_operator(Alu) -> Alu#alu.op.
alu_src2(Alu) -> Alu#alu.src2.
alu_dest_update(Alu,NewDest) -> Alu#alu{dst=NewDest}.
alu_src1_update(Alu,NewSource1) -> Alu#alu{src1=NewSource1}.
alu_operator_update(Alu,NewOp) -> Alu#alu{op=NewOp}.
alu_src2_update(Alu,NewSource2) -> Alu#alu{src2=NewSource2}.
is_alu(Insn) -> case type(Insn) of alu -> true; _ -> false end.

%% Alu_cc %%
alu_cc_create(Dest,Src1,Op,Src2,Info) -> 
  #alu_cc{dst=Dest,src1=Src1,src2=Src2,op=Op,info=Info}.
alu_cc_dest(Alucc) -> Alucc#alu_cc.dst.
alu_cc_src1(Alucc) -> Alucc#alu_cc.src1.
alu_cc_operator(Alucc) -> Alucc#alu_cc.op.
alu_cc_src2(Alucc) -> Alucc#alu_cc.src2.
alu_cc_dest_update(Alu,NewDest) -> Alu#alu_cc{dst=NewDest}.
alu_cc_src1_update(Alu,NewSource1) -> Alu#alu_cc{src1=NewSource1}.
alu_cc_operator_update(Alu,NewOp) -> Alu#alu_cc{op=NewOp}.
alu_cc_src2_update(Alu,NewSource2) -> Alu#alu_cc{src2=NewSource2}.
is_alu_cc(Insn) -> case type(Insn) of alu_cc -> true; _ -> false end.

%% Sethi %%
sethi_create(Dest,Const,Info) -> 
  #sethi{dst=Dest,const=Const,info=Info}.
sethi_dest(Sethi) -> Sethi#sethi.dst.
sethi_const(Sethi) -> Sethi#sethi.const.
sethi_dest_update(SetHi,NewDest) -> SetHi#sethi{dst=NewDest}.
sethi_const_update(SetHi,NewConst) -> SetHi#sethi{const=NewConst}.

%% Load %%
load_create(Dest,Type,Source,Off,Info) -> 
  #load{dst=Dest,type=Type,src=Source,off=Off,info=Info}.
load_dest(Load) -> Load#load.dst.
load_type(Load) -> Load#load.type.
load_src(Load) -> Load#load.src.
load_off(Load) -> Load#load.off.
load_dest_update(Load,NewDest) -> Load#load{dst=NewDest}.
load_type_update(Load,NewType) -> Load#load{type=NewType}.
load_src_update(Load,NewSource) -> Load#load{src=NewSource}.
load_off_update(Load,NewOff) -> Load#load{off=NewOff}.
is_load(Insn) -> case type(Insn) of load -> true; _ -> false end.

%% Store %%
store_create(Target,Off,Type,Source,Info) -> 
  #store{dst=Target,type=Type,src=Source,off=Off,info=Info}.
store_dest(Store) -> Store#store.dst.
store_off(Store) -> Store#store.off.
store_type(Store) -> Store#store.type.
store_src(Store) -> Store#store.src.
store_dest_update(Store,NewTarget) -> Store#store{dst=NewTarget}.
store_off_update(Store,NewOff) -> Store#store{off=NewOff}.
store_type_update(Store,NewType) -> Store#store{type=NewType}.
store_src_update(Store,NewSource) -> Store#store{src=NewSource}.
is_store(Insn) -> case type(Insn) of store -> true; _ -> false end.

%% B %%
b_create(Cond,TrueLabel,FalseLabel,Pred,Annul,Info) -> 
  #b{cc=Cond,true_label=TrueLabel,false_label=FalseLabel, pred=Pred,
     annul=Annul,info=Info}.
b_cond(B) -> B#b.cc.
b_cond_update(B,NewCond) -> B#b{cc=NewCond}.
b_label(B) -> b_true_label(B).
b_label_update(B,NewLabel) -> b_true_label_update(B,NewLabel).
b_true_label(B) -> B#b.true_label.
b_true_label_update(B,NewLabel) -> B#b{true_label=NewLabel}.
b_false_label(B) -> B#b.false_label.
b_false_label_update(B,NewLabel) -> B#b{false_label=NewLabel}.
b_pred(B) -> B#b.pred.
b_pred_update(B,NewPred) -> B#b{pred=NewPred}.
b_annul_update(B,NewAnnul) -> B#b{annul=NewAnnul}.
b_annul(B) -> B#b.annul.
b_taken(B) -> P = b_pred(B), if P > 0.5 -> true; true -> false end.
is_b(Insn) -> case type(Insn) of b -> true; _ -> false end.

%% Br %%
br_create(Reg,RegCond,TrueLabel,FalseLabel,Pred,Annul,Info) -> 
  #br{reg=Reg,rcc=RegCond,true_label=TrueLabel,false_label=FalseLabel,
      pred=Pred,annul=Annul,info=Info}.
br_reg(Br) -> Br#br.reg.
br_reg_update(Br,NewReg) -> Br#br{reg=NewReg}.
br_regcond(Br) -> Br#br.rcc.
br_regcond_update(Br,NewRegCond) -> Br#br{rcc=NewRegCond}.
br_label(Br) -> br_true_label(Br).
br_label_update(Br,NewLabel) -> br_true_label_update(Br,NewLabel).
br_true_label(Br) -> Br#br.true_label.
br_true_label_update(Br,NewLabel) -> Br#br{true_label=NewLabel}.
br_false_label(Br) -> Br#br.false_label.
br_false_label_update(Br,NewLabel) -> Br#br{false_label=NewLabel}.
br_pred(Br) -> Br#br.pred.
br_pred_update(Br,NewPred) -> Br#br{pred=NewPred}.
br_annul(Br) -> Br#br.annul.
br_annul_update(Br,NewAnnul) -> Br#br{annul=NewAnnul}.
br_taken(Br) -> P = br_pred(Br), if P > 0.5 -> true; true -> false end.
is_br(Insn) -> case type(Insn) of br -> true; _ -> false end.


%% Goto %%
goto_create(Label, Info) -> #goto{label=Label, info=Info}.
goto_label(Goto) -> Goto#goto.label.
goto_label_update(Goto, NewLabel) -> Goto#goto{label=NewLabel}.
is_goto(I) -> case type(I) of goto -> true; _ -> false end.


%% Jmp_link %%
jmp_link_create(Target, Offset, Link, Args, Info) -> 
  #jmp_link{target=Target, off=Offset, link=Link, args=Args, info=Info}.
jmp_link_target(JmpLink) -> JmpLink#jmp_link.target.
jmp_link_target_update(JmpLink,NewTarget) -> 
  JmpLink#jmp_link{target=NewTarget}.
jmp_link_off(JmpLink) -> JmpLink#jmp_link.off.
jmp_link_off_update(JmpLink,NewOffset) -> JmpLink#jmp_link{off=NewOffset}.
jmp_link_link(JmpLink) -> JmpLink#jmp_link.link.
jmp_link_link_update(JmpLink,NewLink) -> JmpLink#jmp_link{link=NewLink}.
jmp_link_args(JmpLink) -> JmpLink#jmp_link.args.
jmp_link_args_update(JmpLink,NewArgs) -> JmpLink#jmp_link{args=NewArgs}.
is_jmp_link(Insn) -> case type(Insn) of jmp_link -> true; _ -> false end.

%% Jmp %%
jmp_create(Target, Offset, Args, Info) -> 
  #jmp{target=Target, off=Offset, args=Args, info=Info}.
jmp_create(Target, Offset, Args, Destinations, Info) -> 
  #jmp{target=Target, off=Offset, args=Args, 
       destinations=Destinations, info=Info}.
jmp_target(Jmp) -> Jmp#jmp.target.
jmp_target_update(Jmp,NewTarget) -> Jmp#jmp{target=NewTarget}.
jmp_off(Jmp) -> Jmp#jmp.off.
jmp_off_update(Jmp,NewOffset) -> Jmp#jmp{off=NewOffset}.
jmp_args(Jmp) -> Jmp#jmp.args.
jmp_args_update(Jmp,NewArgs) -> Jmp#jmp{args=NewArgs}.
jmp_destinations(Jmp) -> Jmp#jmp.destinations.
jmp_destinations_update(Jmp,NewDests) -> Jmp#jmp{destinations=NewDests}.

%% Call_link %%
call_link_create(Target,Link,Args,Cont, Fail, Type, Info) ->
  #call_link{target=Target, link=Link, 
	     %% TODO: Fix this when we have multiple retvals.
	     dests=[mk_reg(hipe_sparc_registers:arg(0))],
	     args=Args, 
	     continuation_label=Cont,
	     fail_label=Fail, type = Type, 
	     stack_descriptor=hipe_sparc_stack_descriptors:empty(),
	     info=Info}.
call_link_target(CL) -> CL#call_link.target.
call_link_target_update(CL,NewTarget) -> CL#call_link{target=NewTarget}.
call_link_link(CL) -> CL#call_link.link.
call_link_link_update(CL,NewLink) -> CL#call_link{link=NewLink}.
call_link_args(CL) -> CL#call_link.args.
call_link_dests(CL) -> CL#call_link.dests.
call_link_args_update(CL,NewArgs) -> CL#call_link{args=NewArgs}.
call_link_continuation(I) ->  I#call_link.continuation_label.
call_link_continuation_update(CL,NewC) -> CL#call_link{continuation_label=NewC}.
call_link_fail(I) ->  I#call_link.fail_label.
call_link_fail_update(CL,NewF) -> CL#call_link{fail_label=NewF}.
call_link_type(I) ->  I#call_link.type.
call_link_stack_desc(I) ->  I#call_link.stack_descriptor.
call_link_stack_desc_update(I,NewSD) -> I#call_link{stack_descriptor=NewSD}.
is_call_link(Insn) -> case type(Insn) of call_link -> true; _ -> false end.

%% Integer regs %%
mk_reg(RegNr) -> {sparc_reg, RegNr}.
mk_new_reg() -> mk_reg(hipe_gensym:get_next_var(sparc)).
is_reg(I) -> case I of {sparc_reg, _} -> true; _ -> false end.
reg_nr({sparc_reg, Name}) -> Name.

%% FP regs %%
mk_fpreg(RegNr) -> {sparc_fpreg, RegNr}.
is_fpreg(I) -> case I of {sparc_fpreg, _} -> true; _ -> false end.
fpreg_nr({sparc_fpreg, RegNr}) -> RegNr.

%% Immediates %%
mk_imm(Value) -> {sparc_imm, Value}.
is_imm(I) -> case I of {sparc_imm, _} -> true; _ -> false end.
imm_value({sparc_imm, Value}) -> Value.

%% Spill locations
mk_spill(Pos) -> {spill, Pos}.
is_spill({spill,_}) -> true;
is_spill(_) -> false.
spill_pos({spill, Pos}) -> Pos.
  
  
	
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Selectors
%%

type(Insn) ->
  element(1, Insn).

info(Insn) ->
  if
    record(Insn,label) -> Insn#label.info;
    record(Insn,nop) -> Insn#nop.info;
    record(Insn,align) -> Insn#align.info;
    record(Insn,comment) -> Insn#comment.info;
    record(Insn,move) -> Insn#move.info;
    record(Insn,multimove) -> Insn#multimove.info;
    record(Insn,cmov_cc) -> Insn#cmov_cc.info;
    record(Insn,cmov_r) -> Insn#cmov_r.info;
    record(Insn,alu) -> Insn#alu.info;
    record(Insn,alu_cc) -> Insn#alu_cc.info;
    record(Insn,sethi) -> Insn#sethi.info;
    record(Insn,load) -> Insn#load.info;
    record(Insn,store) -> Insn#store.info;
    record(Insn,b) -> Insn#b.info;
    record(Insn,br) -> Insn#br.info;
    record(Insn,goto) -> Insn#goto.info;
    record(Insn,jmp_link) -> Insn#jmp_link.info;
    record(Insn,jmp) -> Insn#jmp.info;
    record(Insn,load_atom) -> Insn#load_atom.info;
    record(Insn,load_word_index) -> Insn#load_word_index.info;
    record(Insn,load_address) -> Insn#load_address.info;
    record(Insn,call_link) -> Insn#call_link.info;
    record(Insn,pseudo_return) -> Insn#pseudo_return.info;
    record(Insn,pseudo_enter) -> Insn#pseudo_enter.info;
    record(Insn,pseudo_spill) -> Insn#pseudo_spill.info;
    record(Insn,pseudo_unspill) -> Insn#pseudo_unspill.info;
    record(Insn,pseudo_push) -> Insn#pseudo_push.info;
    record(Insn,pseudo_pop) -> Insn#pseudo_pop.info

  end.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Predicates
%%

is_cmov_cc(Insn) -> case type(Insn) of cmov_cc -> true; _ -> false end.
is_sethi(Insn) -> case type(Insn) of  sethi -> true; _ -> false end.
is_jmp(Insn) -> case type(Insn) of jmp -> true; _ -> false end.
%% is_load_atom(Insn) -> case type(Insn) of load_atom -> true; _ -> false end.
%% is_load_address(I) -> case type(I) of load_address -> true; _ -> false end.

is_any_alu(Insn) -> 
  case type(Insn) of 
    alu -> true; 
    alu_cc ->true; 
    _ -> false 
  end.

is_any_cmov(Insn) -> 
  case type(Insn) of
    cmov_cc -> true;
    cmov_r -> true;
    _ -> false
  end.

is_any_memop(Insn) -> 
  case type(Insn) of
    load -> true;
    store -> true;
    _ -> false
  end.

%% These instrs are branches that need delayslot filling.
is_any_branch(Insn) -> 
  case type(Insn) of
    b -> true;
    br -> true;
    goto -> true;
    jmp -> true;
    jmp_link -> true;
    call_link -> true;
    pseudo_return -> true;
    pseudo_enter -> true;
    _ -> false
  end.


%%
%% Changes the target of a jump.
%%

redirect_jmp(Jmp, ToOld, ToNew) ->
  case type(Jmp) of
    br ->
      case br_true_label(Jmp) of
	ToOld ->
	  br_true_label_update(Jmp, ToNew);
	_ ->
	  case br_false_label(Jmp) of
	    ToOld ->
	      br_false_label_update(Jmp, ToNew);
	    _ ->
	      Jmp
	  end
      end;
    b ->
      case b_true_label(Jmp) of
	ToOld ->
	  b_true_label_update(Jmp, ToNew);
	_ ->
	  case b_false_label(Jmp) of
	    ToOld ->
	      b_false_label_update(Jmp, ToNew);
	    _ ->
	      Jmp
	  end
      end;
    goto ->
      case goto_label(Jmp) of
	ToOld ->
	  goto_label_update(Jmp, ToNew);
	_ ->
	  Jmp
      end;
    call_link ->
      case call_link_continuation(Jmp) of
	ToOld ->
	  call_link_continuation_update(Jmp, ToNew);
	_ ->
	  case call_link_fail(Jmp) of
	    ToOld ->
	      call_link_fail_update(Jmp, ToNew);
	    _ ->
	      Jmp
	  end
      end;
    _ ->
      Jmp
  end.


%%
%% The negation of a conditional.
%%

cc_negate(Cond) ->
  case Cond of
    'a'	-> 'n';
    'n'	-> 'a';
    'ne'	-> 'e';
    'e'	-> 'ne';
    'g'	-> 'le';
    'le'	-> 'g';
    'ge'	-> 'l';
    'l'	-> 'ge';
    'gu'	-> 'leu';
    'leu'	-> 'gu';
    'geu'	-> 'lu';
    'lu'	-> 'geu';
    'pos'	-> 'neg';
    'neg'	-> 'pos';
    'vc'	-> 'vs';
    'vs'	-> 'vc'
  end.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Def/Use by instruction.
%%
%% The condition code registers is read and written like any other. 
%% We define a _virtual_ for icc and xcc.
%%

icc_reg() -> mk_reg(hipe_sparc_registers:icc()).
xcc_reg() -> mk_reg(hipe_sparc_registers:xcc()).


%%
%% Returnes a list of registers that are used by an instruction
%%

uses(Instr) -> 
  remove_immediates(all_uses(Instr)).

all_uses(Ins) ->
  case type(Ins) of
    label -> [];
    nop -> [];
    align -> [];
    comment -> [];
    move -> [move_src(Ins)];
    multimove -> multimove_src(Ins);
    cmov_cc -> [icc_reg(), xcc_reg(), cmov_cc_src(Ins)];
    cmov_r -> [cmov_r_src(Ins), cmov_r_reg(Ins)];
    alu -> [alu_src1(Ins), alu_src2(Ins)];
    alu_cc -> [alu_cc_src1(Ins), alu_cc_src2(Ins)];
    sethi -> [sethi_const(Ins)];
    load -> [load_src(Ins), load_off(Ins)];
    store -> [store_dest(Ins), store_src(Ins), store_off(Ins)];
    b -> [icc_reg(), xcc_reg()];
    br -> [br_reg(Ins)];
    goto -> [];
    jmp_link ->
      [jmp_link_target(Ins), jmp_link_off(Ins) | jmp_link_args(Ins)];
    jmp -> [jmp_target(Ins), jmp_off(Ins) | jmp_args(Ins)];
    call_link -> 
      case call_link_type(Ins) of
	closure ->
	  [call_link_target(Ins)|call_link_args(Ins)];
	_ ->
	  call_link_args(Ins)
      end;
    load_atom -> [];
    load_word_index -> [];
    pseudo_return -> pseudo_return_regs(Ins);
    pseudo_enter -> 
      case pseudo_enter_type(Ins) of
	closure ->
	  [pseudo_enter_target(Ins)|
	   pseudo_enter_args(Ins)];
	_ ->
	  pseudo_enter_args(Ins)
      end;
    pseudo_spill -> [pseudo_spill_reg(Ins)];
    pseudo_unspill -> [];
    pseudo_push -> [pseudo_push_reg(Ins)];
    pseudo_pop -> [];
    load_address -> []
  end.

imm_uses(Instr) -> 
  remove_registers(all_uses(Instr)).


%%
%% Returnes a list of registers that are defined by an instruction
%%

defines(Ins) ->
  Defs = case type(Ins) of
	   label -> [];
	   nop -> [];
	   align -> [];
	   comment -> [];
	   move -> [move_dest(Ins)];
	   multimove -> multimove_dest(Ins);
	   cmov_cc -> [cmov_cc_dest(Ins)];
	   cmov_r -> [cmov_r_dest(Ins)];
	   alu -> [alu_dest(Ins)];
	   alu_cc -> [icc_reg(), xcc_reg(), alu_cc_dest(Ins)];
	   sethi -> [sethi_dest(Ins)];
	   load -> [load_dest(Ins)];
	   store -> [];
	   b -> [];
	   br -> [];
	   goto -> [];
	   jmp_link -> [jmp_link_link(Ins)];
	   jmp -> [];
	   call_link -> 
	     [call_link_link(Ins)|
	      call_link_dests(Ins)]; 
	   load_atom -> [load_atom_dest(Ins)];
	   load_word_index -> [load_word_index_dest(Ins)];
	   pseudo_return -> [];
	   pseudo_enter -> [];
	   pseudo_push -> [];
	   pseudo_pop -> [pseudo_pop_reg(Ins)];
	   pseudo_spill -> [];
	   pseudo_unspill -> [pseudo_unspill_reg(Ins)];
	   load_address -> [load_address_dest(Ins)]
	 end,
  remove_immediates(Defs).


%%
%% Totally redundant, but we need it for speed.
%%

def_use(I) ->
  {Def,Use} = 
    case type(I) of
      label -> {[],[]};
      nop -> {[],[]};
      align -> {[],[]};
      comment ->  {[],[]};
      move -> 
	{[move_dest(I)],
	 [move_src(I)]};
      multimove -> 
	{multimove_dest(I),
	 multimove_src(I)};
      cmov_cc ->
	{[cmov_cc_dest(I)],
	 [icc_reg(), xcc_reg(), cmov_cc_src(I)]};
      cmov_r ->
	{[cmov_r_dest(I)],
	 [cmov_r_src(I), cmov_r_reg(I)]};
      alu -> 
	{[alu_dest(I)],
	 [alu_src1(I), alu_src2(I)]};
      alu_cc ->
	{[icc_reg(), xcc_reg(), alu_cc_dest(I)],
	 [alu_cc_src1(I), alu_cc_src2(I)]};
      sethi ->
	{[sethi_dest(I)], 
	 [sethi_const(I)]};
      load -> 
	{[load_dest(I)],
	 [load_src(I), load_off(I)]};
      store ->
	{[],
	 [store_dest(I), store_src(I), store_off(I)]};
      b -> 
	{[],
	 [icc_reg(), xcc_reg()]};
      br -> 
	{[],
	 [br_reg(I)]};
      goto -> {[],[]};
      jmp_link ->
	{[jmp_link_link(I)],
	 [jmp_link_target(I), jmp_link_off(I) | jmp_link_args(I)]};
      jmp ->
	{[],
	 [jmp_target(I), jmp_off(I) | jmp_args(I)]};
      call_link ->
	{
	[call_link_link(I)|
	   call_link_dests(I)],
	case call_link_type(I) of
	  closure -> 
	    [call_link_target(I)|call_link_args(I)];
	  _-> call_link_args(I)
	end};
      load_atom ->
	{[load_atom_dest(I)],
	 []};
      load_word_index ->
	{[load_word_index_dest(I)],
	 []};
     pseudo_return ->
	{[],
	 pseudo_return_regs(I)};
     pseudo_enter ->
	{[],
	 case pseudo_enter_type(I) of
	   closure ->
	     [pseudo_enter_target(I)|
	      pseudo_enter_args(I)];
	   _ ->
	     pseudo_enter_args(I)
	 end};
       pseudo_spill ->
	{[],
	 [pseudo_spill_reg(I)]};
      pseudo_unspill ->
	{[pseudo_unspill_reg(I)],
	 []};
      pseudo_push ->
	{[],
	 [pseudo_push_reg(I)]};
      pseudo_pop ->
	{[pseudo_pop_reg(I)],
	 []};
      load_address -> 
	{[load_address_dest(I)],
	 []}
    end,
  {remove_immediates(Def), remove_immediates(Use)}.


						%
						% Remove immediates from a list
						%

remove_immediates([]) -> 
  [];
remove_immediates([I|Is]) ->
  case is_imm(I) of
    true -> remove_immediates(Is);
    false -> [I | remove_immediates(Is)]
  end.


						%
						% Remove registers from a list
						%

remove_registers([]) -> 
  [];
remove_registers([I|Is]) ->
  case is_reg(I) of
    true -> remove_registers(Is);
    false -> [I | remove_registers(Is)]
  end.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Substitution: replace occurrences of X by Y if {X,Y} is in Subst.
%%

subst(Ins, Subst) ->
  case type(Ins) of
    label -> Ins;
    nop -> Ins;
    align -> Ins;
    comment -> Ins;
    b -> Ins;
    goto -> Ins;
    sethi ->
      NewDst = subst1(Subst, sethi_dest(Ins)),
      sethi_dest_update(Ins, NewDst);
    br ->
      NewReg = subst1(Subst, br_reg(Ins)),
      br_reg_update(Ins, NewReg);
    call_link ->
      Ins1 =
	case call_link_type(Ins) of
	  closure ->
	    NewTarget = subst1(Subst, call_link_target(Ins)),
	    call_link_target_update(Ins, NewTarget);
	  _ -> Ins
	end,

      NewArgs = subst_list(Subst, call_link_args(Ins1)),
      Ins2 =  call_link_args_update(Ins1,NewArgs),
      NewLink = subst1(Subst, call_link_link(Ins2)),
      call_link_link_update(Ins2, NewLink);
    load_atom ->
      NewDst = subst1(Subst, load_atom_dest(Ins)),
      load_atom_dest_update(Ins, NewDst);
    load_word_index ->
      NewDst = subst1(Subst, load_word_index_dest(Ins)),
      load_word_index_dest_update(Ins, NewDst);
    load_address ->
      NewDst = subst1(Subst, load_address_dest(Ins)),
      load_address_dest_update(Ins, NewDst);
    move ->
      NewSrc = subst1(Subst, move_src(Ins)),
      NewDst = subst1(Subst, move_dest(Ins)),
      I0 = move_dest_update(Ins, NewDst),
      move_src_update(I0, NewSrc);
    multimove ->
      NewSrc = subst_list(Subst, multimove_src(Ins)),
      NewDst = subst_list(Subst, multimove_dest(Ins)),
      I0 = multimove_dest_update(Ins, NewDst),
      multimove_src_update(I0, NewSrc);
    cmov_cc ->
      NewSrc = subst1(Subst, cmov_cc_src(Ins)),
      NewDst = subst1(Subst, cmov_cc_dest(Ins)),
      I0 = cmov_cc_dest_update(Ins, NewDst),
      cmov_cc_src_update(I0, NewSrc);
    cmov_r ->
      NewSrc = subst1(Subst, cmov_r_src(Ins)),
      NewReg = subst1(Subst, cmov_r_reg(Ins)),
      NewDst = subst1(Subst, cmov_r_dest(Ins)),
      I0 = cmov_r_dest_update(Ins, NewDst),
      I1 = cmov_r_src_update(I0, NewSrc),
      cmov_r_reg_update(I1, NewReg);
    alu ->
      NewSrc1 = subst1(Subst, alu_src1(Ins)),
      NewSrc2 = subst1(Subst, alu_src2(Ins)),
      NewDst = subst1(Subst, alu_dest(Ins)),
      I0 = alu_dest_update(Ins, NewDst),
      I1 = alu_src1_update(I0, NewSrc1),
      alu_src2_update(I1, NewSrc2);
    alu_cc ->
      NewSrc1 = subst1(Subst, alu_cc_src1(Ins)),
      NewSrc2 = subst1(Subst, alu_cc_src2(Ins)),
      NewDst = subst1(Subst, alu_cc_dest(Ins)),
      I0 = alu_cc_dest_update(Ins, NewDst),
      I1 = alu_cc_src1_update(I0, NewSrc1),
      alu_cc_src2_update(I1, NewSrc2);
    load ->
      NewSrc = subst1(Subst, load_src(Ins)),
      NewOff = subst1(Subst, load_off(Ins)),
      NewDst = subst1(Subst, load_dest(Ins)),
      I0 = load_dest_update(Ins, NewDst),
      I1 = load_src_update(I0, NewSrc),
      load_off_update(I1, NewOff);
    store ->
      NewSrc = subst1(Subst, store_src(Ins)),
      NewOff = subst1(Subst, store_off(Ins)),
      NewDst = subst1(Subst, store_dest(Ins)),
      I0 = store_dest_update(Ins, NewDst),
      I1 = store_src_update(I0, NewSrc),
      store_off_update(I1, NewOff);
    jmp_link ->
      NewTarget = subst1(Subst, jmp_link_target(Ins)),
      NewOff = subst1(Subst, jmp_link_off(Ins)),
      NewLink = subst1(Subst, jmp_link_link(Ins)),
      I0 = jmp_link_link_update(Ins, NewLink),
      I1 = jmp_link_target_update(I0, NewTarget),
      jmp_link_off_update(I1, NewOff);
    pseudo_return ->
      NewRegs = subst_list(Subst, pseudo_return_regs(Ins)),
      pseudo_return_regs_update(Ins, NewRegs);
    pseudo_enter ->
      NewI = 
	case pseudo_enter_type(Ins) of
	  closure ->
	    pseudo_enter_target_update(Ins,
				       subst1(Subst,pseudo_enter_target(Ins)));
	  _ ->
	    Ins
	end,
      NewRegs = subst_list(Subst, pseudo_enter_args(NewI)),
      pseudo_enter_args_update(NewI, NewRegs);
    pseudo_spill ->
      NewReg = subst1(Subst, pseudo_spill_reg(Ins)),
      pseudo_spill_reg_update(Ins, NewReg);
    pseudo_unspill ->
      NewReg = subst1(Subst, pseudo_unspill_reg(Ins)),
      pseudo_unspill_reg_update(Ins, NewReg);
     pseudo_push ->
      NewReg = subst1(Subst, pseudo_push_reg(Ins)),
      pseudo_push_reg_update(Ins, NewReg);
    pseudo_pop ->
      NewReg = subst1(Subst, pseudo_pop_reg(Ins)),
      pseudo_pop_reg_update(Ins, NewReg);
    jmp ->
      NewTarget = subst1(Subst, jmp_target(Ins)),
      NewOff = subst1(Subst, jmp_off(Ins)),
      I0 = jmp_target_update(Ins, NewTarget),
      jmp_off_update(I0, NewOff)
  end.


subst_defines(Ins, Subst) ->
  case type(Ins) of
    label -> Ins;
    nop -> Ins;
    align -> Ins;
    comment -> Ins;
    b -> Ins;
    goto -> Ins;
    jmp -> Ins;
    br -> Ins;
    sethi ->
      NewDst = subst1(Subst, sethi_dest(Ins)),
      sethi_dest_update(Ins, NewDst);
    call_link ->
      NewLink = subst1(Subst, call_link_link(Ins)),
      call_link_link_update(Ins, NewLink);
    load_atom ->
      NewDst = subst1(Subst, load_atom_dest(Ins)),
      load_atom_dest_update(Ins, NewDst);
    load_word_index ->
      NewDst = subst1(Subst, load_word_index_dest(Ins)),
      load_word_index_dest_update(Ins, NewDst);
    load_address ->
      NewDst = subst1(Subst, load_address_dest(Ins)),
      load_address_dest_update(Ins, NewDst);
    move ->
      NewDst = subst1(Subst, move_dest(Ins)),
      move_dest_update(Ins, NewDst);
    multimove ->
      NewDst = subst_list(Subst, multimove_dest(Ins)),
      multimove_dest_update(Ins, NewDst);
    cmov_cc ->
      NewDst = subst1(Subst, cmov_cc_dest(Ins)),
      cmov_cc_dest_update(Ins, NewDst);
    cmov_r ->
      NewDst = subst1(Subst, cmov_r_dest(Ins)),
      cmov_r_dest_update(Ins, NewDst);
    alu ->
      NewDst = subst1(Subst, alu_dest(Ins)),
      alu_dest_update(Ins, NewDst);
    alu_cc ->
      NewDst = subst1(Subst, alu_cc_dest(Ins)),
      alu_cc_dest_update(Ins, NewDst);
    load ->
      NewDst = subst1(Subst, load_dest(Ins)),
      load_dest_update(Ins, NewDst);
    store -> Ins;
    pseudo_return -> Ins;
    pseudo_enter -> Ins;
    pseudo_spill -> Ins;
    pseudo_unspill -> 
      NewReg = subst1(Subst, pseudo_unspill_reg(Ins)),
      pseudo_unspill_reg_update(Ins, NewReg);
    pseudo_push -> Ins;
    pseudo_pop ->
      NewReg = subst1(Subst, pseudo_pop_reg(Ins)),
      pseudo_pop_reg_update(Ins, NewReg);

    jmp_link ->
      NewLink = subst1(Subst, jmp_link_link(Ins)),
      jmp_link_link_update(Ins, NewLink)
  end.


subst_uses(Ins, Subst) ->
  case type(Ins) of
    label -> Ins;
    nop -> Ins;
    align -> Ins;
    comment -> Ins;
    b -> Ins;
    goto -> Ins;
    sethi -> Ins;
    call_link -> 
      case call_link_type(Ins) of
	closure ->
	  NewTarget = subst1(Subst, call_link_target(Ins)),
	  call_link_target_update(Ins, NewTarget);
	_ -> Ins
      end;
    load_atom -> Ins;
    load_word_index -> Ins;
    load_address -> Ins;
    br ->
      NewReg = subst1(Subst, br_reg(Ins)),
      br_reg_update(Ins, NewReg);
    move ->
      NewSrc = subst1(Subst, move_src(Ins)),
      move_src_update(Ins, NewSrc);
    multimove ->
      NewSrc = subst_list(Subst, multimove_src(Ins)),
      multimove_src_update(Ins, NewSrc);
    cmov_cc ->
      NewSrc = subst1(Subst, cmov_cc_src(Ins)),
      cmov_cc_src_update(Ins, NewSrc);
    cmov_r ->
      NewSrc = subst1(Subst, cmov_r_src(Ins)),
      NewReg = subst1(Subst, cmov_r_reg(Ins)),
      I1 = cmov_r_src_update(Ins, NewSrc),
      cmov_r_reg_update(I1, NewReg);
    alu ->
      NewSrc1 = subst1(Subst, alu_src1(Ins)),
      NewSrc2 = subst1(Subst, alu_src2(Ins)),
      I1 = alu_src1_update(Ins, NewSrc1),
      alu_src2_update(I1, NewSrc2);
    alu_cc ->
      NewSrc1 = subst1(Subst, alu_cc_src1(Ins)),
      NewSrc2 = subst1(Subst, alu_cc_src2(Ins)),
      I1 = alu_cc_src1_update(Ins, NewSrc1),
      alu_cc_src2_update(I1, NewSrc2);
    load ->
      NewSrc = subst1(Subst, load_src(Ins)),
      NewOff = subst1(Subst, load_off(Ins)),
      I1 = load_src_update(Ins, NewSrc),
      load_off_update(I1, NewOff);
    store ->
      NewSrc = subst1(Subst, store_src(Ins)),
      NewOff = subst1(Subst, store_off(Ins)),
      NewDst = subst1(Subst, store_dest(Ins)),
      I0 = store_dest_update(Ins, NewDst),
      I1 = store_src_update(I0, NewSrc),
      store_off_update(I1, NewOff);
    jmp_link ->
      NewTarget = subst1(Subst, jmp_link_target(Ins)),
      NewOff = subst1(Subst, jmp_link_off(Ins)),
      I1 = jmp_link_target_update(Ins, NewTarget),
      jmp_link_off_update(I1, NewOff);
    pseudo_return ->
      NewRegs = subst_list(Subst, pseudo_return_regs(Ins)),
      pseudo_return_regs_update(Ins, NewRegs);
    pseudo_enter ->
      NewI = 
	case pseudo_enter_type(Ins) of
	  closure ->
	    pseudo_enter_target_update(Ins,
				       subst1(Subst,pseudo_enter_target(Ins)));
	  _ ->
	    Ins
	end,
      NewRegs = subst_list(Subst, pseudo_enter_args(NewI)),
      pseudo_enter_args_update(NewI, NewRegs);
    pseudo_spill ->
      NewReg = subst1(Subst, pseudo_spill_reg(Ins)),
      pseudo_spill_reg_update(Ins, NewReg);
    pseudo_unspill ->
      Ins;
    pseudo_push ->
      NewReg = subst1(Subst, pseudo_push_reg(Ins)),
      pseudo_push_reg_update(Ins, NewReg);
    pseudo_pop ->
      Ins;
    jmp ->
      NewTarget = subst1(Subst, jmp_target(Ins)),
      NewOff = subst1(Subst, jmp_off(Ins)),
      I0 = jmp_target_update(Ins, NewTarget),
      jmp_off_update(I0, NewOff)
  end.

subst_list(S,Xs) ->
  [subst1(S,X) || X <- Xs].

subst1([],X) -> 
  X;
subst1([{X,Y}|Xs],X) -> 
  Y;
subst1([_|Xs],X) -> 
  subst1(Xs,X).






%% ---------------------------------------------

highest_reg(Code) ->
  highest_reg(Code,0).

highest_reg([I|Is],Max) ->
  Defs = defines(I),
  Uses = uses(I),
  highest_reg(Is,new_max(Defs++Uses,Max));
highest_reg([],Max) ->
  Max.

new_max([V|Vs],Max) ->
  VName = reg_nr(V),
  if VName > Max ->
      new_max(Vs, VName);
     true ->
      new_max(Vs, Max)
  end;
new_max([],Max) -> Max.

%% ----------------------------------------

check_var_range(Sparc) ->
  Code = sparc_code(Sparc),
  RMax = highest_reg(Code),
  {Low, High} = sparc_var_range(Sparc),
  RMax =< High.
