%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
%%%----------------------------------------------------------------------
%%% Purpose   : Interface module for the process 'rshell_server'
%%%----------------------------------------------------------------------

%%---------------------------------------------------------------------------
%%
%% The process implemented by this 'gen_server' callback module is 
%% responsible for the administration of the connections like
%% access rights.
%%
%%---------------------------------------------------------------------------
%%
%% APPLICATION PROCESS STRUCTURE
%%
%% The top of the application under the application master is a supervisor
%% that has the 'one_for_all' property (XX: Why not 'one_for_one'?).
%% 
%%
%%                         +---------------+
%%                         |  rshell_sup  |
%%                         +---------------+
%%                            |   |   |
%%           +----------------+   |   +-------------------+
%%           |                    |                       |
%%           |                    |                       |
%%     +------------+       +------------+        +---------------+
%%    /              \     /              \       | Worker superv |
%%   | rshell_server  |   | rshell_listener |      +---------------+
%%    \              /     \              /           |   |   | |
%%     +------------+       +------------+      +-----+   |   . .
%%                                              |         |
%%                                            +---+     +---+  
%%                                           /     \   /     \ . . . .
%%                                 Workers   \     /   \     / 
%%                                            +---+     +---+  
%%
%% The processes are:
%%
%% Reg.name.       Module                Description
%% ---------------------------------------------------------------------
%% -               rshell                Interface functions, no process def.
%% rshell_server   rshell_server         The administrative process.
%% -               rshell_listener       Waits for a connection and spawns
%%                                       connection handling processes.
%% -               rshell_connection_sup Supervisor for the workers
%% -               rshell_connection     Represent the connection
%% 
%%---------------------------------------------------------------------------

%%---------------------------------------------------------------------------
%% 
%% APPLICATION FUNCTIONAL DESCRIPTION
%%
%% When the 'rshell_listener' get a socket connection initiated from
%% client it starts a "connection" process that represent the connection.
%% 
%%---------------------------------------------------------------------------

%%
%% This process is started by the 'rshell_sup' supervisor.
%%

-module(rshell).

%% Interface functions
%-export([]). 

%% Internal application interface
-export([dump_state/0]).

%%---------------------------------------------------------------------------
%% Exported functions (interface)
%%---------------------------------------------------------------------------

%%---------------------------------------------------------------------------
%% Application interface functions
%%---------------------------------------------------------------------------

dump_state() ->
    gen_server:cast(rshell_server, dump_state).

%%--------------------------------EOF----------------------------------------








