%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
%%%----------------------------------------------------------------------
%%% Purpose   : Application top supervisor
%%%----------------------------------------------------------------------

-module(rshell_sup).

-behaviour(supervisor).

-export([start/0,
	 init/1]).

start() ->
    supervisor:start_link({local, rshell_sup}, rshell_sup, []).


init([]) ->

    % If one goes down we are in trouble, restart all.

    SupFlags = {one_for_all,10,3600},

    % This is the named process 'rshell_server' that take care of all
    % administrative data, like permissions.

    ChildSpec1 = {rshell_server,
		  {rshell_server,start_server,[]},
		  permanent,
		  brutal_kill,
		  worker,
		  [rshell_app,rshell,rshell_server]},

    % This process is waiting for connections and spawns new rshell connection
    % handles when there is a new connection from the client side.

    ChildSpec2 = {rshell_listener,
		  {rshell_listener,start,[]},
		  permanent,
		  brutal_kill,
		  worker,
		  [rshell_listener]},

    % Last child is supervisor for the remporary connection handling processes

    ChildSpec3 = {rshell_connection_sup,
		  {rshell_connection_sup,start,[]},
		  permanent,
		  infinity,
		  supervisor,
		  [rshell_connection_sup]},
    {ok, {SupFlags, [ChildSpec1, ChildSpec2, ChildSpec3]}}.

