%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
%%%----------------------------------------------------------------------
%%% Purpose : Output editing commands and chars
%%%----------------------------------------------------------------------

-module(rshell_terminal).

-compile(export_all).
-export([send_cmd/2]).

%% VT100/XTERM codes
%%
%% Clear tab stops (0 = cursor position, 2 or 3 = all)	ESC [ * g
%% Deselect auto carriage return			ESC [ 20 l
%% Disable cursor					ESC [ ? 50 l
%% Disable cursor autowrap				ESC [ ? 7 l
%% Enable cursor                                        ESC [ ? 50 h
%% Enable cursor autowrap                               ESC [ ? 7 h
%% Index cursor (move down one line)                    ESC D
%% Insert FF character & advance cursor                 FF
%% Line feed   %                                        LF
%% Move cursor down * lines                             ESC [ * B
%% Move cursor down one line                            VT
%% Move cursor left * columns                           ESC [ * D
%% Move cursor one column left                          BS
%% Move cursor right * columns                          ESC [ * C
%% Move cursor to beginning of next line                ESC E
%% Move cursor to left margin of current line           CR
%% Move cursor to line (*l) column (*c)                 ESC [ *l ;  *c H
%% Move cursor to line (*l) column (*c)                 ESC [ *l ; *c f
%% Move cursor to next tab stop                         HT
%% Move cursor up * lines                               ESC [ * A
%% Reverse index cursor (move up one line)              ESC M
%% Select absolute origin mode                          ESC [ ? 6 l
%% Select auto carriage return                          ESC [ 20 h
%% Select relative origin mode                          ESC [ ? 6 h
%% Set tab stop at current cursor position              ESC H
%% Tab cursor backward * tabs                           ESC [ * Z
%% Tab cursor forward * tabs                            ESC [ * I

%% Delete * characters from cursor position right       ESC [ * P
%% Select insert mode                                   ESC [ 4 h


send_cmd(0, Bytes) ->				% put_chars Chars
    Bytes;
send_cmd(1, [N1,N0]) ->				% move_rel int16
    send_pos(signed16(N1,N0));
send_cmd(2, Bytes) ->				% insert_chars Chars
    ["\e[4h",Bytes];
send_cmd(3, [N1,N0]) ->				% delete_chars int16
    send_del(signed16(N1,N0));
send_cmd(4, []) ->				% beep
    [7].

%% Move the cursor

send_pos(0) ->
    "";
send_pos(Pos) when Pos < 0 ->
    io_lib:format("\e[~wD",[-Pos]);
send_pos(Pos) ->
    io_lib:format("\e[~wC",[Pos]).

%% Delete from cursor

send_del(0) ->
    "";
send_del(Pos) when Pos < 0 ->
    io_lib:format("\e[~wD\e[~wP",[-Pos,-Pos]);
send_del(Pos) ->
    io_lib:format("\e[~wP",[Pos]).


signed16(N1,N0) ->
    Unsigned = N1 * 256 + N0,
    SignedMax = 16#7fff,
    if
	Unsigned > SignedMax ->
	    -(256 * 256 - Unsigned);
	true ->
	    Unsigned
    end.
